#include "pgraph.h"

// 
image *reflection(const image *img)
{
	uint w = img->width, h = img->height;
	image *new_img = create_image(w, h);
	const color *pixels = img->pixels;
	color *new_pixels = new_img->pixels;
	for (uint i = 0; i < h; i++)
		for (uint j = 0; j < w; j++)
			new_pixels[i * w + j] = pixels[(i + 1) * w - j];
	return new_img;
}

// 
image *negative(const image *img)
{
	uint w = img->width, h = img->height;
	image *new_img = create_image(w, h);
	const color *pixels = img->pixels;
	color *new_pixels = new_img->pixels;
	for (uint i = 0; i < h; i++)
		for (uint j = 0; j < w; j++)
			new_pixels[i * w + j] = to_color(16777215 - from_color(pixels[i * w + j]));
	return new_img;
}

//
image *black_white(const image *img)
{
	uint w = img->width, h = img->height;
	image *new_img = create_image(w, h);
	const color *pixels = img->pixels;
	color *new_pixels = new_img->pixels;
	for (uint i = 0; i < h; i++)
		for (uint j = 0; j < w; j++)
		{
			color col  = pixels[i * w + j];
			uchar new_col = round(0.3 * col.red + 0.59 * col.green +
								  0.11 * col.blue);
			new_pixels[i * w + j] = (color) {new_col, new_col, new_col};
		}
	return new_img;
}

//  
image *rgb_to_gbr(const image *img)
{
	uint w = img->width, h = img->height;
	image *new_img = create_image(w, h);
	const color *pixels = img->pixels;
	color *new_pixels = new_img->pixels;
	for (uint i = 0; i < h; i++)
		for (uint j = 0; j < w; j++)
		{
			color col  = pixels[i * w + j];
			new_pixels[i * w + j] = (color) {col.green, col.blue, col.red};
		}
	return new_img;
}

int main()
{
	image *img;
	if (!(img = load_from_file("test.bmp")))
		printf("   ");
	else
	{
		image *new_img = reflection(img);
		save_to_file(new_img, "test1.bmp");
		free(new_img);
		new_img = negative(img);
		save_to_file(new_img, "test2.bmp");
		free(new_img);
		new_img = black_white(img);
		save_to_file(new_img, "test3.bmp");
		free(new_img);
		new_img = rgb_to_gbr(img);
		save_to_file(new_img, "test4.bmp");
		image *new_img2 = rgb_to_gbr(new_img);
		save_to_file(new_img2, "test5.bmp");
		free(new_img);
        free(new_img2);			
	}
	free(img);
	return 0;
}
